package io.github.gretzky.connectivity

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.net.ConnectivityManager

import com.facebook.react.bridge.ReactApplicationContext

import io.github.gretzky.connectivity.types.CellularGeneration
import io.github.gretzky.connectivity.types.ConnectionType

class RNDUConnectivityBroadcastReceiver(reactContext: ReactApplicationContext) : RNDUConnectivityReceiver(reactContext) {

    private val mConnectivityReceiverBroadcastReceiver: ConnectivityBroadcastReceiver

    init {
        mConnectivityReceiverBroadcastReceiver = ConnectivityBroadcastReceiver()
    }

    override fun register() {
        val filter = IntentFilter()
        filter.addAction(ConnectivityManager.CONNECTIVITY_ACTION)
        mReactContext.registerReceiver(mConnectivityReceiverBroadcastReceiver, filter)
        mConnectivityReceiverBroadcastReceiver.isRegistered = true
        updateAndSendConnectionType()
    }

    override fun unregister() {
        if (mConnectivityReceiverBroadcastReceiver.isRegistered) {
            mReactContext.unregisterReceiver(mConnectivityReceiverBroadcastReceiver)
            mConnectivityReceiverBroadcastReceiver.isRegistered = false
        }
    }

    private fun updateAndSendConnectionType() {
        var connectionType = ConnectionType.UNKNOWN
        var cellularGeneration = CellularGeneration.UNKNOWN
        var isInternetReachable = false

        try {
            val networkInfo = mConnectivityManager.getActiveNetworkInfo()
            if (!networkInfo!!.isConnected()) {
                connectionType = ConnectionType.NONE
            } else {
                // Check if the internet is reachable
                isInternetReachable = networkInfo.isConnected()

                // Get the connection type
                val networkType = networkInfo.getType()
                when (networkType) {
                    ConnectivityManager.TYPE_MOBILE, ConnectivityManager.TYPE_MOBILE_DUN -> {
                        connectionType = ConnectionType.CELLULAR
                        cellularGeneration = CellularGeneration.fromNetworkInfo(networkInfo)
                        updateCellularAttributes(cellularGeneration, isRoaming, cellService, cellSignal)
                    }
                    ConnectivityManager.TYPE_WIFI -> {
                        connectionType = ConnectionType.WIFI
                        updateWifiAttributes(ssid, ipAddress, wifiStrength)
                    }
                }
            }
        } catch (e: SecurityException) {
            connectionType = ConnectionType.UNKNOWN
        }

        updateConnectivity(connectionType, isInternetReachable, isConnectionExpensive)
    }

    private inner class ConnectivityBroadcastReceiver : BroadcastReceiver() {

        var isRegistered = false

        override fun onReceive(context: Context, intent: Intent) {
            val action = intent.getAction()
            if (action != null && action.equals(ConnectivityManager.CONNECTIVITY_ACTION)) {
                updateAndSendConnectionType()
            }
        }
    }
}
