package io.github.gretzky.connectivity.types

import android.net.NetworkInfo
import android.telephony.TelephonyManager

import javax.annotation.Nullable;

// Based on the EffectiveConnectionType enum described in the W3C Network Information API spec
// (https://wicg.github.io/netinfo/).
enum class CellularGeneration private constructor(val label: String?) {
    // We need to prefix these with "CG_" because they cannot start with numbers
    CG_2G("2g"),
    CG_3G("3g"),
    CG_4G("4g"),
    UNKNOWN(null);


    companion object {

        fun fromNetworkInfo(networkInfo: NetworkInfo?): CellularGeneration {
            if (networkInfo == null) {
                return CellularGeneration.UNKNOWN
            }

            when (networkInfo!!.getSubtype()) {
                TelephonyManager.NETWORK_TYPE_1xRTT, TelephonyManager.NETWORK_TYPE_CDMA, TelephonyManager.NETWORK_TYPE_EDGE, TelephonyManager.NETWORK_TYPE_GPRS, TelephonyManager.NETWORK_TYPE_IDEN -> return CellularGeneration.CG_2G
                TelephonyManager.NETWORK_TYPE_EHRPD, TelephonyManager.NETWORK_TYPE_EVDO_0, TelephonyManager.NETWORK_TYPE_EVDO_A, TelephonyManager.NETWORK_TYPE_EVDO_B, TelephonyManager.NETWORK_TYPE_HSDPA, TelephonyManager.NETWORK_TYPE_HSPA, TelephonyManager.NETWORK_TYPE_HSUPA, TelephonyManager.NETWORK_TYPE_UMTS -> return CellularGeneration.CG_3G
                TelephonyManager.NETWORK_TYPE_HSPAP, TelephonyManager.NETWORK_TYPE_LTE -> return CellularGeneration.CG_4G
                TelephonyManager.NETWORK_TYPE_UNKNOWN -> return CellularGeneration.UNKNOWN
                else -> return CellularGeneration.UNKNOWN
            }
        }
    }
}
