export declare enum ConnectivityType {
    CELLULAR = "cellular",
    WIFI = "wifi",
    UNKNOWN = "unknown",
    NONE = "none"
}
export declare enum CellularGeneration {
    "2g" = "2g",
    "3g" = "3g",
    "4g" = "4g"
}
export interface ConnectivityDetails {
    strength: number;
    isConnectionExpensive: boolean;
    isInternetReachable: boolean;
}
export interface TypeConnectivityState<T extends ConnectivityType, D extends object = {}> {
    type: T;
    isConnected: boolean;
    details: D & ConnectivityDetails;
}
export declare type CellularConnectivity = TypeConnectivityState<ConnectivityType.CELLULAR, {
    cellularGeneration: CellularGeneration | null;
    carrier: string;
    isInService: boolean;
    isRoaming: boolean;
}>;
export declare type WifiConnectivity = TypeConnectivityState<ConnectivityType.WIFI, {
    ssid: string;
    ip: string;
}>;
export declare type UnknownConnectivity = TypeConnectivityState<ConnectivityType.UNKNOWN>;
export declare type NoConnectivity = TypeConnectivityState<ConnectivityType.NONE>;
export declare type ConnectivityInfo = CellularConnectivity | WifiConnectivity | UnknownConnectivity | NoConnectivity;
export interface ConnectivityInterface {
    connectivity: ConnectivityInfo;
}
export interface ConnectivityComponentProps {
    state: ConnectivityInterface;
    color?: string;
}
export declare type ConnectivityState = ConnectivityInterface | null;
export declare type ConnectivityChangeHandler = (state: ConnectivityState) => void;
export declare type ConnectivitySubscription = () => void;
export interface ConnectivityModule {
    getCurrentState: () => Promise<ConnectivityState>;
    addListener: (type: string, handler: Function) => void;
    removeListeners: (type: string, handler: Function) => void;
}
