import { NativeModules, NativeEventEmitter } from "react-native";
import Listener from "./listener";
import { ConnectivityModule, ConnectivityState, ConnectivityChangeHandler, ConnectivitySubscription } from "./types";

const ConnectivityNativeModule: ConnectivityModule = NativeModules.RNDUConnectivity;

if (!ConnectivityNativeModule) {
  throw new Error(`@react-native-device-utils/connectivity: Native Module is null. Steps to fix:

• Run \`react-native-link @react-native-device-utils/connectivity\` in the project root
• Rebuild and run the app
• Manually link the library if necessary
• If you're getting this error while testing, you didn't read the README. #shame

If none of these fix the issue, open an issue on Github: https://github.com/react-native-device-utils.git`);
}

let nativeEventEmitter: NativeEventEmitter | null = null;

const ConnectivityInterface = {
  ...ConnectivityNativeModule,
  get eventEmitter(): NativeEventEmitter | null {
    if (!nativeEventEmitter) {
      /// @ts-ignore
      nativeEventEmitter = new NativeEventEmitter(ConnectivityNativeModule);
    }

    return nativeEventEmitter;
  },

  addEventListener(listener: ConnectivityChangeHandler): ConnectivitySubscription {
    Listener.add(listener);
    return (): void => {
      Listener.remove(listener);
    };
  },

  getCurrentState(): Promise<ConnectivityState> {
    return ConnectivityNativeModule.getCurrentState();
  }
};

export default ConnectivityInterface;
