import { NativeEventSubscription } from "react-native";
import ConnectivityInterface from "./interface";
import { ConnectivityChangeHandler, ConnectivityState } from "./types";

const _subscriptions = new Set<ConnectivityChangeHandler>();
let _nativeEventSubscription: NativeEventSubscription | null = null;
let _latestState: ConnectivityState = null;

function _listenerHandler(state: ConnectivityState): void {
  _latestState = state;
  _subscriptions.forEach((handler): void => handler(state));
}

function add(handler: ConnectivityChangeHandler, latestOnListen: boolean = true): void {
  _subscriptions.add(handler);

  if (latestOnListen) {
    if (_latestState) {
      handler(_latestState);
    } else {
      ConnectivityInterface.getCurrentState().then((state: ConnectivityState): void => {
        _latestState = state;
        handler(_latestState);
      });
    }
  }

  if (_subscriptions.size > 0 && !_nativeEventSubscription) {
    /// @ts-ignore
    _nativeEventSubscription = ConnectivityInterface.eventEmitter.addListener("connectivity", _listenerHandler);
  }
}

function clear(): void {
  _subscriptions.clear();

  if (_nativeEventSubscription) {
    _nativeEventSubscription.remove();
    _nativeEventSubscription = null;
  }
}

function remove(handler: any): void {
  _subscriptions.delete(handler);

  if (_subscriptions.size === 0 && _nativeEventSubscription) {
    _nativeEventSubscription.remove();
    _nativeEventSubscription = null;
  }
}

export default {
  add,
  remove,
  clear
};
