export enum ConnectivityType {
  CELLULAR = "cellular",
  WIFI = "wifi",
  UNKNOWN = "unknown",
  NONE = "none"
}

export enum CellularGeneration {
  "2g" = "2g",
  "3g" = "3g",
  "4g" = "4g"
}

export interface ConnectivityDetails {
  strength: number;
  isConnectionExpensive: boolean;
  isInternetReachable: boolean;
}

export interface TypeConnectivityState<T extends ConnectivityType, D extends object = {}> {
  type: T;
  isConnected: boolean;
  details: D & ConnectivityDetails;
}

export type CellularConnectivity = TypeConnectivityState<
  ConnectivityType.CELLULAR,
  {
    cellularGeneration: CellularGeneration | null;
    carrier: string;
    isInService: boolean;
    isRoaming: boolean;
  }
>;

export type WifiConnectivity = TypeConnectivityState<
  ConnectivityType.WIFI,
  {
    ssid: string;
    ip: string;
  }
>;

export type UnknownConnectivity = TypeConnectivityState<ConnectivityType.UNKNOWN>;

export type NoConnectivity = TypeConnectivityState<ConnectivityType.NONE>;

export type ConnectivityInfo = CellularConnectivity | WifiConnectivity | UnknownConnectivity | NoConnectivity;

export interface ConnectivityInterface {
  connectivity: ConnectivityInfo;
}

export interface ConnectivityComponentProps {
  state: ConnectivityInterface;
  color?: string;
}

export type ConnectivityState = ConnectivityInterface | null;
export type ConnectivityChangeHandler = (state: ConnectivityState) => void;
export type ConnectivitySubscription = () => void;

export interface ConnectivityModule {
  getCurrentState: () => Promise<ConnectivityState>;
  addListener: (type: string, handler: Function) => void;
  removeListeners: (type: string, handler: Function) => void;
}
